@@ -1,3 +1,16 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004-2008 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #ifndef __CMAN_TOOL_DOT_H__
 #define __CMAN_TOOL_DOT_H__
 
@@ -63,7 +76,9 @@ struct commandline
         char *interfaces[MAX_INTERFACES];
 	char *override_nodename;
 	char *key_filename;
+	char *filename;
 	char *format_opts;
+	char *config_lcrso;
 	int votes;
 	int expected_votes;
 	int two_node;
@@ -88,6 +103,7 @@ struct commandline
 	int fence_opt;
 	int addresses_opt;
 	int noccs_opt;
+	int nosetpri_opt;
 };
 typedef struct commandline commandline_t;
 
