@@ -1,3 +1,16 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004-2008 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <sys/wait.h>
 #include <stdint.h>
 #include <signal.h>
@@ -38,76 +51,6 @@ static void be_daemon(int close_stderr)
 	setsid();
 }
 
-static char *aisexec_exit_reason(signed char status)
-{
-	static char reason[256];
-	switch (status) {
-	case -2:
-		return "Could not determine UID to run as";
-		break;
-	case -3:
-		return "Could not determine GID to run as";
-		break;
-	case -4:
-		return "Error initialising memory pool";
-		break;
-	case -5:
-		return "Could not fork";
-		break;
-	case -6:
-		return "Could not bind to libais socket";
-		break;
-	case -7:
-		return "Could not bind to network socket";
-		break;
-	case -8:
-		return "Could not read security key for communications";
-		break;
-	case -9:
-		return "Could not read cluster configuration";
-		break;
-	case -10:
-		return "Could not set up logging";
-		break;
-	case -11:
-		return "Could not dynamically load modules";
-		break;
-	case -12:
-		return "Could not load and initialise object database";
-		break;
-	case -13:
-		return "Could not initialise all required services";
-		break;
-	case -14:
-		return "Out of memory";
-		break;
-	default:
-		sprintf(reason, "Error, reason code is %d", status);
-		return reason;
-		break;
-	}
-}
-
-static int check_aisexec_status(pid_t pid)
-{
-	int status;
-	int pidstatus;
-
-	status = waitpid(pid, &pidstatus, WNOHANG);
-	if (status == -1 && errno == ECHILD) {
-
-		return 0;
-	}
-	if ((status == 0 || status == pid) && pidstatus != 0) {
-		if (WIFEXITED(pidstatus))
-			fprintf(stderr, "aisexec died: %s\n", aisexec_exit_reason(WEXITSTATUS(pidstatus)));
-		if (WIFSIGNALED(pidstatus))
-			fprintf(stderr, "aisexec died with signal: %d\n", WTERMSIG(pidstatus));
-		exit(1);
-	}
-	return status;
-}
-
 int join(commandline_t *comline)
 {
 	int i;
@@ -178,10 +121,12 @@ int join(commandline_t *comline)
 	}
 	if (comline->noccs_opt) {
 		envp[envptr++] = strdup("CMAN_NOCCS=true");
+		envp[envptr++] = strdup("OPENAIS_DEFAULT_CONFIG_IFACE=cmanpreconfig");
+	}
+	else {
+		snprintf(scratch, sizeof(scratch), "OPENAIS_DEFAULT_CONFIG_IFACE=%s:cmanpreconfig", comline->config_lcrso);
+		envp[envptr++] = strdup(scratch);
 	}
-
-	/* Use cman to configure services */
-	envp[envptr++] = strdup("OPENAIS_DEFAULT_CONFIG_IFACE=cmanconfig");
 
 	/* Create a pipe to monitor cman startup progress */
 	pipe(p);
@@ -190,10 +135,11 @@ int join(commandline_t *comline)
 	envp[envptr++] = strdup(scratch);
 	envp[envptr++] = NULL;
 
-	/* Always run aisexec -f because we have already forked twice anyway, and
-	   we want to return any exit code that might happen */
 	argv[0] = "aisexec";
-	argv[++argvptr] = "-f";
+	if (comline->verbose & ~DEBUG_STARTUP_ONLY)
+		argv[++argvptr] = "-f";
+	if (comline->nosetpri_opt)
+		argv[++argvptr] = "-p";
 	argv[++argvptr] = NULL;
 
 	/* Fork/exec cman */
@@ -215,10 +161,6 @@ int join(commandline_t *comline)
 			}
 		}
 		be_daemon(!(comline->verbose & ~DEBUG_STARTUP_ONLY));
-
-		sprintf(scratch, "FORKED: %d\n", getpid());
-		write(p[1], scratch, strlen(scratch));
-
 		execve(AISEXECBIN, argv, envp);
 
 		/* exec failed - tell the parent process */
@@ -242,44 +184,49 @@ int join(commandline_t *comline)
 		fd_set fds;
 		struct timeval tv={1, 0};
 		char message[1024];
-		char *messageptr = message;
 
 		FD_ZERO(&fds);
 		FD_SET(p[0], &fds);
 
 		status = select(p[0]+1, &fds, NULL, NULL, &tv);
 
-		/* Did we get a cman-reported error? */
+		/* Did we get an error? */
 		if (status == 1) {
 			int len;
 			if ((len = read(p[0], message, sizeof(message)) > 0)) {
 
-				/* Forked OK - get the real aisexec pid */
-				if (sscanf(messageptr, "FORKED: %d", &aisexec_pid) == 1) {
-					if (comline->verbose & DEBUG_STARTUP_ONLY)
-						fprintf(stderr, "forked process ID is %d\n", aisexec_pid);
-					status = 1;
-
-					/* There might be a SUCCESS message in the pipe too. */
-					messageptr = strchr(messageptr, '\n');
-					if (messageptr)
-						messageptr++;
-					else
-						continue;
-				}
 				/* Success! get the new PID of double-forked aisexec */
-				if (sscanf(messageptr, "SUCCESS: %d", &aisexec_pid) == 1) {
+				if (sscanf(message, "SUCCESS: %d", &aisexec_pid) == 1) {
 					if (comline->verbose & DEBUG_STARTUP_ONLY)
 						fprintf(stderr, "aisexec running, process ID is %d\n", aisexec_pid);
 					status = 0;
-					break;
 				}
+				else {
+					fprintf(stderr, "cman not started: %s\n", message);
+				}
+				break;
 			}
 			else if (len < 0 && errno == EINTR) {
 				continue;
 			}
 			else { /* Error or EOF - check the child status */
-				status = check_aisexec_status(aisexec_pid);
+				int pidstatus;
+				status = waitpid(aisexec_pid, &pidstatus, WNOHANG);
+				if (status == -1 && errno == ECHILD) {
+					fprintf(stderr, "cman not started\n");
+					break;
+				}
+				if (status == 0 && pidstatus != 0) {
+					if (WIFEXITED(pidstatus))
+						fprintf(stderr, "aisexec died with status: %d\n", WEXITSTATUS(pidstatus));
+					if (WIFSIGNALED(pidstatus))
+						fprintf(stderr, "aisexec died with signal: %d\n", WTERMSIG(pidstatus));
+					status = -1;
+					break;
+				}
+				else {
+					status = 0; /* Try to connect */
+				}
 			}
 		}
 
@@ -291,15 +238,13 @@ int join(commandline_t *comline)
 		do {
 			if (status == 0) {
 				if (kill(aisexec_pid, 0) < 0) {
-					status = check_aisexec_status(aisexec_pid);
 					die("aisexec died during startup\n");
 				}
 
 				h = cman_admin_init(NULL);
 				if (!h && comline->verbose & DEBUG_STARTUP_ONLY)
 				{
-					fprintf(stderr, "waiting for cman to start\n");
-					status = check_aisexec_status(aisexec_pid);
+					fprintf(stderr, "waiting for aisexec to start\n");
 				}
 			}
 			sleep (1);
