@@ -1,3 +1,16 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004-2008 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <inttypes.h>
 #include <unistd.h>
 #include <signal.h>
@@ -7,7 +20,9 @@
 #include "libcman.h"
 #include "cman_tool.h"
 
-#define OPTION_STRING		("m:n:v:e:2p:c:r:i:N:t:o:k:F:Vwfqah?Xd::")
+#define DEFAULT_CONFIG_MODULE "ccsconfig"
+
+#define OPTION_STRING		("m:n:v:e:2p:c:r:i:N:t:o:k:F:C:VPwfqah?Xd::")
 #define OP_JOIN			1
 #define OP_LEAVE		2
 #define OP_EXPECTED		3
@@ -19,6 +34,7 @@
 #define OP_NODES		9
 #define OP_SERVICES		10
 #define OP_DEBUG		11
+#define OP_DUMP_OBJDB		12
 
 
 static void print_usage(int subcmd)
@@ -46,10 +62,12 @@ static void print_usage(int subcmd)
 		printf("  -n <nodename>    The name of this node (defaults to hostname)\n");
 		printf("  -c <clustername> The name of the cluster to join\n");
 		printf("  -N <id>          Node id\n");
+		printf("  -C <module>      Config file reader (default: ccsconfig)\n");
 		printf("  -w               Wait until node has joined a cluster\n");
 		printf("  -q               Wait until the cluster is quorate\n");
 		printf("  -t               Maximum time (in seconds) to wait\n");
 		printf("  -k <file>        Private key file for AIS communications\n");
+		printf("  -P               Don't set aisexec to realtime priority\n");
 		printf("  -X               Don't use CCS for configuration\n");
 		printf("\n");
 	}
@@ -219,7 +237,6 @@ static void show_status(void)
 	if (cman_get_quorum_device(h, &qinfo) == 0 && qinfo.qi_state == 2)
 		printf("Quorum device votes: %d\n", qinfo.qi_votes);
 	printf("Total votes: %d\n", einfo->ei_total_votes);
-	printf("Node votes: %d\n", einfo->ei_node_votes);
 
 	printf("Quorum: %d %s\n", einfo->ei_quorum, quorate?" ":"Activity blocked");
 	printf("Active subsystems: %d\n", cman_get_subsys_count(h));
@@ -554,18 +571,11 @@ static void leave(commandline_t *comline)
 	}
 
 	result = cman_shutdown(h, flags);
-	if (result && !comline->wait_opt) {
+	if (result) {
 		die("Error leaving cluster: %s", cman_error(errno));
 	}
-	cman_finish(h);
 
-	/* Wait until cman shuts down */
-	if (comline->wait_opt) {
-		while ( (h = cman_admin_init(NULL)) ) {
-			cman_finish(h);
-			sleep(1);
-		}
-	}
+	cman_finish(h);
 }
 
 static void set_expected(commandline_t *comline)
@@ -699,6 +709,19 @@ static void set_debuglog(commandline_t *comline)
 	cman_finish(h);
 }
 
+#ifdef DEBUG
+static void dump_objdb(commandline_t *comline)
+{
+	cman_handle_t h;
+
+	h = open_cman_handle(1);
+
+	if (cman_dump_objdb(h, comline->filename))
+		perror("dump objdb failed");
+
+	cman_finish(h);
+}
+#endif
 
 static int get_int_arg(char argopt, char *arg)
 {
@@ -722,6 +745,8 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 	int optchar, i;
 	int show_help = 0;
 
+	comline->config_lcrso=DEFAULT_CONFIG_MODULE;
+
 	while (cont) {
 		optchar = getopt(argc, argv, OPTION_STRING);
 
@@ -759,6 +784,10 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 			comline->key_filename = strdup(optarg);
 			break;
 
+		case 'C':
+			comline->config_lcrso = strdup(optarg);
+			break;
+
 		case 'r':
 			comline->config_version = get_int_arg(optchar, optarg);
 			comline->config_version_opt = TRUE;
@@ -843,6 +872,10 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 		case 'X':
 			comline->noccs_opt = TRUE;
 			break;
+
+		case 'P':
+			comline->nosetpri_opt = TRUE;
+			break;
 		default:
 			die("unknown option: %c", optchar);
 			break;
@@ -894,6 +927,18 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 			if (comline->operation)
 				die("can't specify two operations");
 			comline->operation = OP_DEBUG;
+#ifdef DEBUG
+		} else if (strcmp(argv[optind], "dump-db") == 0) {
+			if (comline->operation)
+				die("can't specify two operations");
+			comline->operation = OP_DUMP_OBJDB;
+			if (!argv[optind+1])
+				die("no filename given");
+			comline->filename = strdup(argv[optind+1]);
+			if (comline->filename[0] != '/')
+				die("dump filename must be an absolute path");
+			optind++;
+#endif
 		} else if (strcmp(argv[optind], "remove") == 0) {
 			comline->remove = TRUE;
 		} else if (strcmp(argv[optind], "force") == 0) {
@@ -1012,6 +1057,11 @@ int main(int argc, char *argv[])
 	case OP_DEBUG:
 		set_debuglog(&comline);
 		break;
+#ifdef DEBUG
+	case OP_DUMP_OBJDB:
+		dump_objdb(&comline);
+		break;
+#endif
 	}
 
 	exit(EXIT_SUCCESS);
