@@ -1,9 +1,21 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004-2008 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <getopt.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <inttypes.h>
 #include <syslog.h>
 #include <string.h>
 #include <sys/time.h>
@@ -17,11 +29,14 @@
 #include <arpa/inet.h>
 #include <netinet/in.h>
 #include <sys/errno.h>
+#include <dlfcn.h>
 
+#include <openais/service/objdb.h>
 #include <openais/service/swab.h>
 #include <openais/totem/totemip.h>
 #include <openais/totem/totempg.h>
 #include <openais/service/swab.h>
+#include <openais/service/logsys.h>
 #include <openais/service/timer.h>
 #include <openais/totem/aispoll.h>
 #include "list.h"
@@ -30,7 +45,7 @@
 #include "daemon.h"
 #include "barrier.h"
 #include "logging.h"
-#include "cmanccs.h"
+#include "cmanconfig.h"
 #include "commands.h"
 #include "ais.h"
 
@@ -64,6 +79,7 @@ static uint16_t cluster_id;
 static int ais_running;
 static time_t join_time;
 static openais_timer_handle quorum_device_timer;
+static struct objdb_iface_ver0 *global_objdb;
 
 /* If CCS gets out of sync, we poll it until it isn't */
 static openais_timer_handle ccsd_timer;
@@ -202,7 +218,7 @@ static struct cluster_node *add_new_node(char *name, int nodeid, int votes, int
 	if (!newnode) {
 		newnode = malloc(sizeof(struct cluster_node));
 		if (!newnode) {
-			// TODO what ??
+			log_printf(LOG_ERR, "Unable to allocate memory for node %s\n", name);
 			return NULL;
 		}
 		memset(newnode, 0, sizeof(struct cluster_node));
@@ -243,6 +259,8 @@ static struct cluster_node *add_new_node(char *name, int nodeid, int votes, int
 	if (newalloc)
 		node_add_ordered(newnode);
 
+	newnode->flags |= NODE_FLAGS_REREAD;
+
 	P_MEMB("add_new_node: %s, (id=%d, votes=%d) newalloc=%d\n",
 	       name, nodeid, votes, newalloc);
 
@@ -323,7 +341,7 @@ static void recalculate_quorum(int allow_decrease, int by_current_nodes)
 
 	quorum = calculate_quorum(allow_decrease, by_current_nodes?cluster_members:0, &total_votes);
 	set_quorate(total_votes);
-	notify_listeners(NULL, EVENT_REASON_STATECHANGE, cluster_is_quorate);
+	notify_listeners(NULL, EVENT_REASON_STATECHANGE, 0);
 }
 
 /* Copy internal node format to userland format */
@@ -375,7 +393,9 @@ int cman_set_nodeid(int nodeid)
 	return 0;
 }
 
-int cman_join_cluster(char *name, unsigned short cl_id, int two_node_flag, int expected_votes)
+int cman_join_cluster(struct objdb_iface_ver0 *objdb,
+		      char *name, unsigned short cl_id,
+		      int two_node_flag, int votes, int expected_votes)
 {
 	if (ais_running)
 		return -EALREADY;
@@ -386,15 +406,11 @@ int cman_join_cluster(char *name, unsigned short cl_id, int two_node_flag, int e
 	cluster_id = cl_id;
 	strncpy(cluster_name, name, MAX_CLUSTER_NAME_LEN);
 	two_node = two_node_flag;
+	global_objdb = objdb;
 
 	quit_threads = 0;
 	ais_running = 1;
 
-	if (read_ccs_nodes(&config_version, 1)) {
-		log_printf(LOG_ERR, "Can't initialise list of nodes from CCS\n");
-		return -EINVAL;
-	}
-
 	/* Make sure we have a node name */
 	if (nodename[0] == '\0') {
 		struct utsname un;
@@ -403,7 +419,7 @@ int cman_join_cluster(char *name, unsigned short cl_id, int two_node_flag, int e
 	}
 
 	time(&join_time);
-	us = add_new_node(nodename, wanted_nodeid, 1, expected_votes,
+	us = add_new_node(nodename, wanted_nodeid, votes, expected_votes,
 			  NODESTATE_DEAD);
 	set_port_bit(us, 0);
 	us->us = 1;
@@ -437,10 +453,15 @@ static int do_cmd_get_extrainfo(char *cmdbuf, char **retbuf, int retsize, int *r
 {
 	char *outbuf = *retbuf + offset;
 	struct cl_extra_info *einfo = (struct cl_extra_info *)outbuf;
+	struct totem_ip_address node_ifs[MAX_INTERFACES];
 	int total_votes = 0;
 	int max_expected = 0;
 	int addrlen;
 	int uncounted = 0;
+	unsigned int num_interfaces;
+	unsigned int totem_object_handle;
+	unsigned int object_handle;
+	char **status;
 	struct cluster_node *node;
 	struct sockaddr_storage *ss;
 	char *ptr;
@@ -449,6 +470,8 @@ static int do_cmd_get_extrainfo(char *cmdbuf, char **retbuf, int retsize, int *r
 	if (!we_are_a_cluster_member)
 		return -ENOENT;
 
+	totempg_ifaces_get(us->node_id, node_ifs, &status, &num_interfaces);
+
 	list_iterate_items(node, &cluster_members_list) {
 		if (node->state == NODESTATE_MEMBER) {
 			total_votes += node->votes;
@@ -462,9 +485,9 @@ static int do_cmd_get_extrainfo(char *cmdbuf, char **retbuf, int retsize, int *r
 
         /* Enough room for addresses ? */
 	if (retsize < (sizeof(struct cl_extra_info) +
-		       sizeof(struct sockaddr_storage) * (num_interfaces*2))) {
+		       sizeof(struct sockaddr_storage) * (MAX_INTERFACES*2))) {
 
-		*retbuf = malloc(sizeof(struct cl_extra_info) + sizeof(struct sockaddr_storage) * (num_interfaces*2));
+		*retbuf = malloc(sizeof(struct cl_extra_info) + sizeof(struct sockaddr_storage) * (MAX_INTERFACES*2));
 		outbuf = *retbuf + offset;
 		einfo = (struct cl_extra_info *)outbuf;
 
@@ -492,15 +515,39 @@ static int do_cmd_get_extrainfo(char *cmdbuf, char **retbuf, int retsize, int *r
 		einfo->flags |= CMAN_EXTRA_FLAG_DIRTY;
 
 	ptr = einfo->addresses;
-	for (i=0; i<num_interfaces; i++) {
-		ss = (struct sockaddr_storage *)ptr;
-		totemip_totemip_to_sockaddr_convert(&mcast_addr[i], 0, ss, &addrlen);
-		ptr += sizeof(struct sockaddr_storage);
+
+	global_objdb->object_find_reset(OBJECT_PARENT_HANDLE);
+	if (global_objdb->object_find(OBJECT_PARENT_HANDLE,
+				      "totem", strlen("totem"),
+				      &totem_object_handle) == 0) {
+
+		while (global_objdb->object_find(totem_object_handle,
+						 "interface", strlen("interface"),
+						 &object_handle) == 0) {
+
+			char *mcast;
+			struct sockaddr_in *saddr4;
+			struct sockaddr_in6 *saddr6;
+
+			objdb_get_string(global_objdb, object_handle, "mcastaddr", &mcast);
+			memset(ptr, 0, sizeof(struct sockaddr_storage));
+
+			saddr4 = (struct sockaddr_in *)ptr;
+			saddr6 = (struct sockaddr_in6 *)ptr;
+			if ( inet_pton(AF_INET, mcast, &saddr4->sin_addr) >0) {
+				saddr4->sin_family = AF_INET;
+			}
+			else {
+				if (inet_pton(AF_INET6, mcast, &saddr6->sin6_addr) > 0)
+					saddr4->sin_family = AF_INET6;
+			}
+			ptr += sizeof(struct sockaddr_storage);
+		}
 	}
 
 	for (i=0; i<num_interfaces; i++) {
 		ss = (struct sockaddr_storage *)ptr;
-		totemip_totemip_to_sockaddr_convert(&ifaddrs[i], 0, ss, &addrlen);
+		totemip_totemip_to_sockaddr_convert(&node_ifs[i], 0, ss, &addrlen);
 		ptr += sizeof(struct sockaddr_storage);
 	}
 
@@ -922,7 +969,7 @@ static int do_cmd_try_shutdown(struct connection *con, char *cmdbuf)
 	int flags = *(int *)cmdbuf;
 
 	/* Are we already in shutdown ? */
-	if (shutdown_con || quit_threads)
+	if (shutdown_con)
 		return -EALREADY;
 
 	shutdown_con = con;
@@ -933,7 +980,6 @@ static int do_cmd_try_shutdown(struct connection *con, char *cmdbuf)
 
 	/* If no-one is listening for events then we can just go down now */
 	if (shutdown_expected == 0) {
-		quit_threads = 1;
 		send_leave(CLUSTER_LEAVEFLAG_DOWN);
 		return 0;
 	}
@@ -1018,7 +1064,7 @@ static void ccsd_timer_fn(void *arg)
 	int ccs_err;
 
 	log_printf(LOG_DEBUG, "Polling ccsd for updated information\n");
-	ccs_err = read_ccs_nodes(&config_version, 0);
+	ccs_err = read_cman_nodes(global_objdb, &config_version, 0);
 	if (ccs_err || config_version < wanted_config_version) {
 		log_printf(LOG_ERR, "Can't read CCS to get updated config version %d. Activity suspended on this node\n",
 				wanted_config_version);
@@ -1162,11 +1208,8 @@ static int do_cmd_get_node_addrs(char *cmdbuf, char **retbuf, int retsize, int *
 	memset(outbuf, 0, retsize - offset);
 
 	/* AIS doesn't know about nodes that are not members */
-	if (node->state != NODESTATE_MEMBER)  {
-		addrs->numaddrs = 0;
-		*retlen = sizeof(struct cl_get_node_addrs);
+	if (node->state != NODESTATE_MEMBER)
 		return 0;
-	}
 
 	if (totempg_ifaces_get(nodeid, node_ifs, &status, (unsigned int *)&addrs->numaddrs))
 		return -errno;
@@ -1177,7 +1220,7 @@ static int do_cmd_get_node_addrs(char *cmdbuf, char **retbuf, int retsize, int *
 						    &addrs->addrs[i].addrlen);
 	}
 	*retlen = sizeof(struct cl_get_node_addrs) +
-		num_interfaces * sizeof(struct cl_node_addrs);
+		addrs->numaddrs * sizeof(struct cl_node_addrs);
 
 	return 0;
 }
@@ -1189,6 +1232,9 @@ int process_command(struct connection *con, int cmd, char *cmdbuf,
 	struct cl_version cnxman_version;
 	char *outbuf = *retbuf;
 	int value;
+#ifdef DEBUG
+	FILE *dumpfile;
+#endif
 
 	P_MEMB("command to process is %x\n", cmd);
 
@@ -1219,7 +1265,20 @@ int process_command(struct connection *con, int cmd, char *cmdbuf,
 		set_debuglog(value);
 		err = 0;
 		break;
-
+#ifdef DEBUG
+	case CMAN_CMD_DUMP_OBJDB:
+		P_MEMB("CC: Dump_objdb: filename = %s\n", cmdbuf);
+		dumpfile = fopen(cmdbuf, "w+");
+		if (dumpfile)  {
+			global_objdb->object_dump(OBJECT_PARENT_HANDLE, dumpfile);
+			fclose(dumpfile);
+			err = 0;
+		}
+		else {
+			err = -errno;
+		}
+		break;
+#endif
 	case CMAN_CMD_START_CONFCHG:
 		con->confchg = 1;
 		err = 0;
@@ -1454,11 +1513,6 @@ void unbind_con(struct connection *con)
 
 		check_shutdown_status();
 	}
-
-	/* If the controlling shutdown process has quit, then cancel the
-	   shutdown session */
-	if (con == shutdown_con)
-		shutdown_con = NULL;
 }
 
 /* Post a PORT OPEN/CLOSE event to anyone listening on this end */
@@ -1504,7 +1558,7 @@ static int valid_transition_msg(int nodeid, struct cl_transmsg *msg)
 	if (msg->config_version > config_version) {
 		int ccs_err;
 
-		ccs_err = read_ccs_nodes(&config_version, 0);
+		ccs_err = read_cman_nodes(global_objdb, &config_version, 0);
 		if (ccs_err || config_version < msg->config_version) {
 			config_error = 1;
 			log_printf(LOG_ERR, "Can't read CCS to get updated config version %d. Activity suspended on this node\n",
@@ -1515,7 +1569,8 @@ static int valid_transition_msg(int nodeid, struct cl_transmsg *msg)
 						   ccsd_timer_fn, &ccsd_timer);
 		}
 		if (config_version > msg->config_version) {
-			// TODO tell everyone else to update...
+			/* Tell everyone else to update */
+			send_reconfigure(us->node_id, RECONFIG_PARAM_CONFIG_VERSION, config_version);
 		}
 		recalculate_quorum(0, 0);
 	}
@@ -1542,7 +1597,7 @@ void send_transition_msg(int last_memb_count, int first_trans)
 	P_MEMB("sending TRANSITION message. cluster_name = %s\n", cluster_name);
 	msg->cmd = CLUSTER_MSG_TRANSITION;
 	msg->first_trans = first_trans;
-	msg->votes = us->votes;
+	msg->high_nodeid = get_highest_nodeid();
 	msg->expected_votes = us->expected_votes;
 	msg->cluster_id = cluster_id;
 	msg->major_version = CNXMAN_MAJOR_VERSION;
@@ -1592,7 +1647,7 @@ static void byteswap_internal_message(char *data, int len)
 	case CLUSTER_MSG_TRANSITION:
 		transmsg = (struct cl_transmsg *)data;
 		transmsg->cluster_id = swab16(transmsg->cluster_id);
-		transmsg->votes = swab32(transmsg->votes);
+		transmsg->high_nodeid = swab32(transmsg->high_nodeid);
 		transmsg->expected_votes = swab32(transmsg->expected_votes);
 		transmsg->major_version = swab32(transmsg->major_version);
 		transmsg->minor_version = swab32(transmsg->minor_version);
@@ -1663,9 +1718,10 @@ static void do_reconfigure_msg(void *data)
 		break;
 
 	case RECONFIG_PARAM_CONFIG_VERSION:
-		if (read_ccs_nodes(&config_version, 0)) {
+		if (config_version != msg->value &&
+		    read_cman_nodes(global_objdb, &config_version, 0)) {
 			log_printf(LOG_ERR, "Can't read CCS to get updated config version %d. Activity suspended on this node\n",
-				msg->value);
+				   msg->value);
 
 			config_error = 1;
 			recalculate_quorum(0, 0);
@@ -1709,7 +1765,8 @@ static void do_process_transition(int nodeid, char *data, int len)
 
 	if (valid_transition_msg(nodeid, msg) != 0) {
 		P_MEMB("Transition message from %d does not match current config - should quit ?\n", nodeid);
-		return; // PJC ???
+		// Now what ??
+		return;
 	}
 
 	/* If the remote node can see AISONLY nodes then we can't join as we don't
@@ -1728,13 +1785,10 @@ static void do_process_transition(int nodeid, char *data, int len)
 	}
 	assert(node);
 
-	P_MEMB("Got TRANSITION message. msg->flags=%x, node->flags=%x, first_trans=%d\n",
-	       msg->flags, node->flags, msg->first_trans);
-
 	/* Newer nodes 6.1.0 onwards, set the DIRTY flag if they have state. If the new node has been down
 	   and has state then we mark it disallowed because we cannot merge stateful nodes */
-	if ( (msg->flags & NODE_FLAGS_DIRTY && (node->flags & NODE_FLAGS_BEENDOWN)) ||
-	     (msg->flags & NODE_FLAGS_DIRTY && msg->first_trans && !node->us)) {
+	if (msg->flags & NODE_FLAGS_DIRTY && (node->flags & NODE_FLAGS_BEENDOWN ||
+	    (us->flags & NODE_FLAGS_DIRTY && nodeid != us->node_id))) {
 		/* Don't duplicate messages */
 		if (node->state != NODESTATE_AISONLY) {
 			if (cluster_is_quorate) {
@@ -1754,8 +1808,7 @@ static void do_process_transition(int nodeid, char *data, int len)
 
         /* This is for older nodes. If the join_time of the node matches that already stored AND
 	   the node has been down, then we kill it as this must be a rejoin */
-	if (msg->minor_version == 0 &&
-	    msg->join_time == node->cman_join_time && node->flags & NODE_FLAGS_BEENDOWN) {
+	if (msg->join_time == node->cman_join_time && node->flags & NODE_FLAGS_BEENDOWN) {
 		/* Don't duplicate messages */
 		if (node->state != NODESTATE_AISONLY) {
 			if (cluster_is_quorate) {
@@ -1874,7 +1927,7 @@ static void process_internal_message(char *data, int len, int nodeid, int need_b
 		if (killmsg->nodeid == wanted_nodeid) {
 			/* Must use syslog directly here or the message will never arrive */
 			syslog(LOG_CRIT, "cman killed by node %d because %s\n", nodeid,
-				killmsg_reason(killmsg->reason));
+			       killmsg_reason(killmsg->reason));
 			exit(1);
 		}
 		break;
@@ -1935,6 +1988,32 @@ void override_expected(int newexp)
 	}
 }
 
+void clear_reread_flags()
+{
+	struct list *nodelist;
+	struct cluster_node *node;
+
+	list_iterate(nodelist, &cluster_members_list) {
+		node = list_item(nodelist, struct cluster_node);
+		node->flags &= ~NODE_FLAGS_REREAD;
+	}
+}
+
+void remove_unread_nodes()
+{
+	struct list *nodelist;
+	struct cluster_node *node;
+
+	list_iterate(nodelist, &cluster_members_list) {
+		node = list_item(nodelist, struct cluster_node);
+		if (!(node->flags & NODE_FLAGS_REREAD) &&
+		    node->state == NODESTATE_DEAD) {
+
+			list_del(&node->list);
+			free(node);
+		}
+	}
+}
 
 /* Add a node from CCS, note that it may already exist if user has simply updated the config file */
 void add_ccs_node(char *nodename, int nodeid, int votes, int expected_votes)
@@ -1947,7 +2026,7 @@ void add_ais_node(int nodeid, uint64_t incarnation, int total_members)
 {
 	struct cluster_node *node;
 
-	P_MEMB("add_ais_node ID=%d, incarnation = %" PRIu64 "\n",nodeid, incarnation);
+	P_MEMB("add_ais_node ID=%d, incarnation = %lld\n",nodeid, incarnation);
 
 	node = find_node_by_nodeid(nodeid);
 	if (!node && total_members == 1) {
@@ -1965,7 +2044,7 @@ void add_ais_node(int nodeid, uint64_t incarnation, int total_members)
 		node = add_new_node(tempname, nodeid, 1, total_members, NODESTATE_DEAD);
 	}
 
-	if (node->state == NODESTATE_DEAD || node->state == NODESTATE_LEAVING) {
+	if (node->state == NODESTATE_DEAD) {
 		gettimeofday(&node->join_time, NULL);
 		node->incarnation = incarnation;
 		node->state = NODESTATE_MEMBER;
@@ -2026,7 +2105,6 @@ void del_ais_node(int nodeid)
 	}
 }
 
-// TODO make these more efficient!
 static int get_highest_nodeid()
 {
 	int highest = 0;
