@@ -1,3 +1,22 @@
+/*
+  Copyright Red Hat, Inc. 2002, 2007
+  Copyright Mission Critical Linux, 2000
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2, or (at your option) any
+  later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; see the file COPYING.  If not, write to the
+  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
+  MA 02139, USA.
+*/
 /** @file
  * daemon_init function, does sanity checks and calls daemon().
  *
@@ -23,11 +42,13 @@
 #include <sys/errno.h>
 #include <libgen.h>
 #include <signal.h>
+#include <openais/service/logsys.h>
 
 /*
  * This should ultimately go in a header file.
  */
 void daemon_init(char *prog);
+int check_pid_valid(pid_t pid, char *prog);
 int check_process_running(char *prog, pid_t * pid);
 
 /*
@@ -36,8 +57,9 @@ int check_process_running(char *prog, pid_t * pid);
 static void update_pidfile(char *prog);
 static int setup_sigmask(void);
 
+LOGSYS_DECLARE_SUBSYS ("QDISK", LOG_LEVEL_INFO);
 
-static int
+int
 check_pid_valid(pid_t pid, char *prog)
 {
 	FILE *fp;
@@ -128,13 +150,8 @@ check_process_running(char *prog, pid_t * pid)
 	if (fp == NULL) {	/* error */
 		return 0;
 	}
-
-	ret = fscanf(fp, "%d\n", &oldpid);
+	fscanf(fp, "%d\n", &oldpid);
 	fclose(fp);
-
-	if ((ret == EOF) || (ret != 1))
-		return 0;
-
 	if (check_pid_valid(oldpid, cmd)) {
 		*pid = oldpid;
 		return 1;
@@ -201,27 +218,23 @@ daemon_init(char *prog)
 
 	uid = getuid();
 	if (uid) {
-		fprintf(stderr,
+		log_printf(LOG_ERR,
 			"daemon_init: Sorry, only root wants to run this.\n");
 		exit(1);
 	}
 
 	if (check_process_running(prog, &pid) && (pid != getpid())) {
-		fprintf(stderr,
+		log_printf(LOG_ERR,
 			"daemon_init: Process \"%s\" already running.\n",
 			prog);
 		exit(1);
 	}
 	if (setup_sigmask() < 0) {
-		fprintf(stderr, "daemon_init: Unable to set signal mask.\n");
-		exit(1);
-	}
-
-	if(daemon(0, 0)) {
-		fprintf(stderr, "daemon_init: Unable to daemonize.\n");
+		log_printf(LOG_ERR, "daemon_init: Unable to set signal mask.\n");
 		exit(1);
 	}
 
+	daemon(0, 0);
 
 	update_pidfile(prog);
 }
