@@ -1,3 +1,27 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
+**
+**  This library is free software; you can redistribute it and/or
+**  modify it under the terms of the GNU Lesser General Public
+**  License as published by the Free Software Foundation; either
+**  version 2 of the License, or (at your option) any later version.
+**
+**  This library is distributed in the hope that it will be useful,
+**  but WITHOUT ANY WARRANTY; without even the implied warranty of
+**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+**  Lesser General Public License for more details.
+**
+**  You should have received a copy of the GNU Lesser General Public
+**  License along with this library; if not, write to the Free Software
+**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+**
+*******************************************************************************
+******************************************************************************/
+
+
 #ifdef _REENTRANT
 #include <pthread.h>
 #endif
@@ -23,6 +47,10 @@
 #include "libdlm.h"
 #include <linux/dlm_device.h>
 
+/* this define will come through linux/dlm.h in 2.6.25-rc */
+#ifndef EXPERIMENTAL_BUILD
+#define DLM_LOCKSPACE_LEN 64
+#endif
 
 #define PROC_MISC		"/proc/misc"
 #define MISC_PREFIX		"/dev/misc/"
@@ -1238,32 +1266,48 @@ int dlm_ls_pthread_init(dlm_lshandle_t ls)
 
 static int create_lockspace_v5(const char *name, uint32_t flags)
 {
-	char reqbuf[sizeof(struct dlm_write_request_v5) + strlen(name)];
+	char reqbuf[sizeof(struct dlm_write_request_v5) + DLM_LOCKSPACE_LEN];
 	struct dlm_write_request_v5 *req = (struct dlm_write_request_v5 *)reqbuf;
+	int namelen = strlen(name);
 	int minor;
 
-	req->cmd = DLM_USER_CREATE_LOCKSPACE;
+	memset(reqbuf, 0, sizeof(reqbuf));
 	set_version_v5(req);
-	strcpy(req->i.lspace.name, name);
+
+	req->cmd = DLM_USER_CREATE_LOCKSPACE;
 	req->i.lspace.flags = flags;
 
-	minor = write(control_fd, req, sizeof(*req) + strlen(name));
+	if (namelen > DLM_LOCKSPACE_LEN) {
+		errno = EINVAL;
+		return -1;
+	}
+	memcpy(req->i.lspace.name, name, namelen);
+
+	minor = write(control_fd, req, sizeof(*req) + namelen);
 
 	return minor;
 }
 
 static int create_lockspace_v6(const char *name, uint32_t flags)
 {
-	char reqbuf[sizeof(struct dlm_write_request) + strlen(name)];
+	char reqbuf[sizeof(struct dlm_write_request) + DLM_LOCKSPACE_LEN];
 	struct dlm_write_request *req = (struct dlm_write_request *)reqbuf;
+	int namelen = strlen(name);
 	int minor;
 
-	req->cmd = DLM_USER_CREATE_LOCKSPACE;
+	memset(reqbuf, 0, sizeof(reqbuf));
 	set_version_v6(req);
-	strcpy(req->i.lspace.name, name);
+
+	req->cmd = DLM_USER_CREATE_LOCKSPACE;
 	req->i.lspace.flags = flags;
 
-	minor = write(control_fd, req, sizeof(*req) + strlen(name));
+	if (namelen > DLM_LOCKSPACE_LEN) {
+		errno = EINVAL;
+		return -1;
+	}
+	memcpy(req->i.lspace.name, name, namelen);
+
+	minor = write(control_fd, req, sizeof(*req) + namelen);
 
 	return minor;
 }
