@@ -1,3 +1,21 @@
+/*
+  Copyright Red Hat, Inc. 2006
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2, or (at your option) any
+  later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; see the file COPYING.  If not, write to the
+  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
+  MA 02139, USA.
+*/
 /*
  * @file fence_xvmd.c: Implementation of server daemon for Xen virtual
  * machine fencing.  This uses SA AIS CKPT b.1.0 checkpointing API to 
@@ -104,9 +122,7 @@ tcp_exchange(int fd, fence_auth_type_t auth, void *key,
 		return -1;
 
 	/* Read return code */
-	if (read(fd, &ret, 1) < 0)
-		return -1;
-
+	read(fd, &ret, 1);
 	close(fd);
 	if (ret == 0)
 		printf("Remote: Operation was successful\n");
@@ -314,14 +330,6 @@ main(int argc, char **argv)
 
 	if (args.flags & F_HELP) {
 		args_usage(argv[0], my_options, 0);
-
-                printf("With no command line argument, arguments are "
-                       "read from standard input.\n");
-                printf("Arguments read from standard input take "
-                       "the form of:\n\n");
-                printf("    arg1=value1\n");
-                printf("    arg2=value2\n\n");
-
 		args_usage(argv[0], my_options, 1);
 		exit(0);
 	}
