@@ -1,3 +1,24 @@
+/*
+  Copyright Red Hat, Inc. 2006
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2, or (at your option) any
+  later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; see the file COPYING.  If not, write to the
+  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
+  MA 02139, USA.
+*/
+/*
+ * Author: Lon Hohberger <lhh at redhat.com>
+ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -33,6 +54,8 @@
 #include "libcman.h"
 #include "debug.h"
 
+#define LIBVIRT_XEN_URI "xen:///"
+
 static int running = 1;
 static int reload_key;
 
@@ -199,17 +222,15 @@ wait_domain(fence_req_t *req, virConnectPtr vp, int timeout)
 
 int
 do_fence_request_tcp(fence_req_t *req, fence_auth_type_t auth,
-		     void *key, size_t key_len, virConnectPtr vp,
-		     int flags)
+		     void *key, size_t key_len, virConnectPtr vp)
 {
 	int fd = -1, ret = -1;
-	virDomainPtr vdp = NULL, nvdp = NULL;
-	virDomainInfo vdi;
+	virDomainPtr vdp;
 	char response = 1;
 	char *domain_desc, *domain_desc_sanitized;
 	size_t sz;
 
-	if (!(vdp = get_domain(req, vp)) && (!(flags & F_NOCLUSTER))) {
+	if (!(vdp = get_domain(req, vp))) {
 		dbg_printf(2, "Could not find domain: %s\n", req->domain);
 		goto out;
 	}
@@ -228,17 +249,6 @@ do_fence_request_tcp(fence_req_t *req, fence_auth_type_t auth,
 		break;
 	case FENCE_OFF:
 		printf("Destroying domain %s...\n", (char *)req->domain);
-		if (flags & F_NOCLUSTER) {
-			if (!vdp ||
-			    ((virDomainGetInfo(vdp, &vdi) == 0) &&
-			     (vdi.state == VIR_DOMAIN_SHUTOFF))) {
-				dbg_printf(2, "[NOCLUSTER] Nothing to "
-					   "do - domain does not exist\n");
-				response = 0;
-				break;
-			}
-		}
-
 
 		dbg_printf(2, "[OFF] Calling virDomainDestroy\n");
 		ret = virDomainDestroy(vdp);
@@ -256,18 +266,6 @@ do_fence_request_tcp(fence_req_t *req, fence_auth_type_t auth,
 	case FENCE_REBOOT:
 		printf("Rebooting domain %s...\n",
 		       (char *)req->domain);
-
-		if (flags & F_NOCLUSTER) {
-			if (!vdp ||
-			    ((virDomainGetInfo(vdp, &vdi) == 0) &&
-			     (vdi.state == VIR_DOMAIN_SHUTOFF))) {
-				dbg_printf(2, "[NOCLUSTER] Nothing to "
-					   "do - domain does not exist\n");
-				response = 0;
-				break;
-			}
-		}
-
 		domain_desc = virDomainGetXMLDesc(vdp, 0);
 
 		if (domain_desc) {
@@ -288,10 +286,10 @@ do_fence_request_tcp(fence_req_t *req, fence_auth_type_t auth,
 			       "libvirt\n");
 		}
 
-		dbg_printf(2, "[REBOOT] Calling virDomainDestroy(%p)\n", vdp);
+		dbg_printf(2, "[REBOOT] Calling virDomainDestroy\n");
 		ret = virDomainDestroy(vdp);
 		if (ret < 0) {
-			printf("virDomainDestroy() failed: %d/%d\n", ret, errno);
+			printf("virDomainDestroy() failed: %d\n", ret);
 			if (domain_desc)
 				free(domain_desc);
 			break;
@@ -303,36 +301,19 @@ do_fence_request_tcp(fence_req_t *req, fence_auth_type_t auth,
 			printf("Domain still exists; fencing failed\n");
 		} else if (domain_desc) {
 			/* Recreate the domain if possible */
-			/* Success... or not? */
+			/* Success */
 			dbg_printf(2, "Calling virDomainCreateLinux()...\n");
-			nvdp = virDomainCreateLinux(vp, domain_desc, 0);
-
-			if (nvdp == NULL) {
-				/* More recent versions of libvirt or perhaps the
- 				   KVM back-end do not let you create a domain from
- 				   XML if there is already a defined domain description
- 				   with the same name that it knows about.  You must
- 				   then call virDomainCreate() */
-				dbg_printf(2, "Failed; Trying virDomainCreate()...\n");
-				if (virDomainCreate(vdp) < 0) {
-					printf("Failed to recreate guest %s!\n",
-					       (char *)req->domain);
-				}
-			}
+			virDomainCreateLinux(vp, domain_desc, 0);
 			free(domain_desc);
 		}
 		break;
 	}
-
+	
 	dbg_printf(3, "Sending response to caller...\n");
 	if (write(fd, &response, 1) < 0) {
 		perror("write");
 	}
 out:
-	if (vdp)
-		virDomainFree(vdp);
-	if (nvdp)
-		virDomainFree(nvdp);
 	if (fd != -1)
 		close(fd);
 
@@ -563,7 +544,7 @@ xvmd_loop(cman_handle_t ch, void *h, int fd, fence_xvm_args_t *args,
 	virt_list_t *vl = NULL;
 	virt_state_t *dom = NULL;
 
-	vp = virConnectOpen(args->uri);
+	vp = virConnectOpen(LIBVIRT_XEN_URI);
 	if (!vp)
 		perror("virConnectOpen");
 
@@ -620,7 +601,7 @@ xvmd_loop(cman_handle_t ch, void *h, int fd, fence_xvm_args_t *args,
 			continue;
 	
 		/* Request and/or timeout: open connection */
-		vp = virConnectOpen(args->uri);
+		vp = virConnectOpen(LIBVIRT_XEN_URI);
 		if (!vp) {
 			printf("NOTICE: virConnectOpen(): %s; cannot fence!\n",
 			       strerror(errno));
@@ -695,7 +676,7 @@ xvmd_loop(cman_handle_t ch, void *h, int fd, fence_xvm_args_t *args,
 		case AUTH_SHA512:
 			printf("Plain TCP request\n");
 			do_fence_request_tcp(&data, args->auth, key,
-					     key_len, vp, args->flags);
+					     key_len, vp);
 			break;
 		default:
 			printf("XXX Unhandled authentication\n");
@@ -735,7 +716,7 @@ main(int argc, char **argv)
 	int mc_sock;
 	char key[MAX_KEY_LEN];
 	int key_len = 0, x;
-	char *my_options = "dfi:a:p:I:C:U:c:k:u?hLXV";
+	char *my_options = "dfi:a:p:C:c:k:u?hLXV";
 	cman_handle_t ch = NULL;
 	void *h = NULL;
 
@@ -753,17 +734,12 @@ main(int argc, char **argv)
 	}
 
 	if (args.flags & F_ERR) {
+		args_usage(argv[0], my_options, 0);
 		return 1;
 	}
 
 	if (args.flags & F_HELP) {
 		args_usage(argv[0], my_options, 0);
-
-		printf("Arguments may be specified as part of the\n");
-		printf("fence_xvmd tag in cluster.conf in the form of:\n");
-		printf("    <fence_xvmd argname=\"value\" ... />\n\n");
-
-		args_usage(argv[0], my_options, 1);
 		return 0;
 	}
 
@@ -789,11 +765,7 @@ main(int argc, char **argv)
 	/* XXX need to wait for child to successfully start before
 	   exiting... */
 	if (!(args.flags & F_FOREGROUND))
-		if(daemon(0,0)) {
-			printf("Could not daemonize\n");
-			return 1;
-		}
-			
+		daemon(0,0);
 
 	if (virInitialize() != 0) {
 		printf("Could not initialize libvirt\n");
@@ -836,9 +808,9 @@ main(int argc, char **argv)
 			printf("Checkpoint initialized\n");
 	}
 	if (args.family == PF_INET)
-		mc_sock = ipv4_recv_sk(args.addr, args.port, args.ifindex);
+		mc_sock = ipv4_recv_sk(args.addr, args.port);
 	else
-		mc_sock = ipv6_recv_sk(args.addr, args.port, args.ifindex);
+		mc_sock = ipv6_recv_sk(args.addr, args.port);
 	if (mc_sock < 0) {
 		printf("Could not set up multicast listen socket\n");
 		return 1;
