@@ -35,6 +35,16 @@ int fs_compute_bitstructs(struct fsck_rgrp *rgd)
 	uint32 bytes_left, bytes;
 	int x;
 
+	/* Max size of an rg is 2GB.  A 2GB RG with (minimum) 512-byte blocks
+	   has 4194304 blocks.  We can represent 4 blocks in one bitmap byte.
+	   Therefore, all 4194304 blocks can be represented in 1048576 bytes.
+	   Subtract a metadata header for each 512-byte block and we get
+	   488 bytes of bitmap per block.  Divide 1048576 by 488 and we can
+	   be assured we should never have more than 2149 of them. */
+	if (length > 2149 || length == 0) {
+		log_err("Invalid length %u found in rindex.\n", length);
+		return -1;
+	}
 	if(!(rgd->rd_bits = (fs_bitmap_t *)malloc(length * sizeof(fs_bitmap_t)))) {
 		log_err("Unable to allocate bitmap structure\n");
 		stack;
@@ -141,8 +151,13 @@ struct fsck_rgrp *fs_blk2rgrpd(struct fsck_sb *sdp, uint64 blk)
 	return rgd;
 }
 
-
-int fs_rgrp_read(struct fsck_rgrp *rgd)
+/**
+ * fs_rgrp_read - read in the resource group information from disk.
+ * @rgd - resource group structure
+ * @repair_if_corrupted - If TRUE, rgrps found to be corrupt should be repaired
+ *                 according to the index.  If FALSE, no corruption is fixed.
+ */
+int fs_rgrp_read(struct fsck_rgrp *rgd, int repair_if_corrupted)
 {
 	struct fsck_sb *sdp = rgd->rd_sbd;
 	unsigned int x, length = rgd->rd_ri.ri_length;
@@ -160,20 +175,50 @@ int fs_rgrp_read(struct fsck_rgrp *rgd)
 			exit(1);
 		}
 		error = get_and_read_buf(sdp, rgd->rd_ri.ri_addr + x,
-					 &(rgd->rd_bh[x]), 0);
+								 &(rgd->rd_bh[x]), 0);
 		if (error) {
 		  	log_err("Unable to read rgrp from disk.\n"); 
 		  	goto fail;
 		}
 
 		if(check_meta(rgd->rd_bh[x], (x) ? GFS_METATYPE_RB : GFS_METATYPE_RG)){
-			log_err("Buffer #%"PRIu64" (%d of %d) is neither"
-				" GFS_METATYPE_RB nor GFS_METATYPE_RG.\n",
-				BH_BLKNO(rgd->rd_bh[x]),
-				(int)x+1,
-				(int)length);
-			error = -1;
-			goto fail;
+			log_err("Block #%"PRIu64" (0x%"PRIx64") (%d of %d) is neither"
+					" GFS_METATYPE_RB nor GFS_METATYPE_RG.\n",
+					BH_BLKNO(rgd->rd_bh[x]), BH_BLKNO(rgd->rd_bh[x]),
+					(int)x+1, (int)length);
+			if (repair_if_corrupted) {
+				if (query(sdp, "Fix the RG? (y/n)")) {
+					log_err("Attempting to repair the RG.\n");
+					if (x) {
+						struct gfs_meta_header mh;
+
+						memset(&mh, 0, sizeof(mh));
+						mh.mh_magic = GFS_MAGIC;
+						mh.mh_type = GFS_METATYPE_RB;
+						mh.mh_format = GFS_FORMAT_RB;
+						gfs_meta_header_out(&mh,
+								    BH_DATA(rgd->rd_bh[x]));
+					} else {
+						memset(&rgd->rd_rg, 0,
+						       sizeof(struct gfs_rgrp));
+						rgd->rd_rg.rg_header.mh_magic =
+							GFS_MAGIC;
+						rgd->rd_rg.rg_header.mh_type =
+							GFS_METATYPE_RG;
+						rgd->rd_rg.rg_header.mh_format =
+							GFS_FORMAT_RG;
+						rgd->rd_rg.rg_free =
+							rgd->rd_ri.ri_data;
+						gfs_rgrp_out(&rgd->rd_rg,
+							     BH_DATA(rgd->rd_bh[x]));
+					}
+					write_buf(sdp, rgd->rd_bh[x], BW_WAIT);
+				}
+			}
+			else {
+				error = -1;
+				goto fail;
+			}
 		}
 	}
 
@@ -184,11 +229,13 @@ int fs_rgrp_read(struct fsck_rgrp *rgd)
 
  fail:
 	for (x = 0; x < length; x++){
-		relse_buf(sdp, rgd->rd_bh[x]);
-		rgd->rd_bh[x] = NULL;
+		if (rgd->rd_bh[x]) {
+			relse_buf(sdp, rgd->rd_bh[x]);
+			rgd->rd_bh[x] = NULL;
+		}
 	}
 
-	log_err("Resource group is corrupted.\n");
+	log_err("Resource group or index is corrupted.\n");
 	return error;
 }
 
@@ -201,13 +248,15 @@ void fs_rgrp_relse(struct fsck_rgrp *rgd)
 		log_debug("rgrp still held...\n");
 	} else {
 		for (x = 0; x < length; x++){
-			relse_buf(rgd->rd_sbd, rgd->rd_bh[x]);
-			rgd->rd_bh[x] = NULL;
+			if (rgd->rd_bh[x]) {
+				relse_buf(rgd->rd_sbd, rgd->rd_bh[x]);
+				rgd->rd_bh[x] = NULL;
+			}
 		}
 	}
 }
 
-
+#if 0 /* no one calls this, so don't waste memory for it: */
 /**
  * rgrp_verify - Verify that a resource group is consistent
  * @sdp: the filesystem
@@ -264,7 +313,7 @@ int fs_rgrp_verify(struct fsck_rgrp *rgd)
 	}
 	return 0;
 }
-
+#endif
 
 /**
  * fs_rgrp_recount - adjust block tracking numbers
@@ -438,7 +487,7 @@ int clump_alloc(struct fsck_rgrp *rgd, uint32 goal)
 	log_debug("clump_alloc failing...\n");
 	for(--i; i >=0; i--){
 		fs_set_bitmap(sdp, BH_BLKNO(bh[i]), GFS_BLKST_FREE);
-		relse_buf(sdp, bh[i]);
+		/*relse_buf(sdp, bh[i]);*/
 	}
 	return -1;
 }
@@ -467,7 +516,7 @@ int fs_blkalloc(struct fsck_inode *ip, uint64 *block)
 			return -1;
 		}
 
-		if(fs_rgrp_read(rgd)){
+		if(fs_rgrp_read(rgd, FALSE)){
 			log_err( "fs_blkalloc:  Unable to read rgrp.\n");
 			return -1;
 		}
@@ -540,7 +589,7 @@ int fs_metaalloc(struct fsck_inode *ip, uint64 *block)
 			return -1;
 		}
 
-		if(fs_rgrp_read(rgd)){
+		if(fs_rgrp_read(rgd, FALSE)){
 			log_err( "fs_metaalloc:  Unable to read rgrp.\n");
 			return -1;
 		}
