@@ -1,3 +1,16 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004-2005 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include "bio.h"
 #include "fs_bits.h"
 
@@ -156,7 +169,7 @@ int next_rg_meta(struct fsck_rgrp *rgd, uint64 *block, int first)
   for(; i < length; i++){
     bits = &rgd->rd_bits[i];
 
-    blk = fs_bitfit((unsigned char *)BH_DATA(rgd->rd_bh[i]) + bits->bi_offset,
+    blk = fs_bitfit(BH_DATA(rgd->rd_bh[i]) + bits->bi_offset,
                     bits->bi_len, blk, GFS_BLKST_USEDMETA);
 
     if(blk != BFITNOENT){
@@ -209,10 +222,10 @@ int next_rg_meta_free(struct fsck_rgrp *rgd, uint64 *block, int first, int *mfre
   for(; i < length; i++){
     bits = &rgd->rd_bits[i];
 
-    ublk = fs_bitfit((unsigned char *)BH_DATA(rgd->rd_bh[i]) + bits->bi_offset,
+    ublk = fs_bitfit(BH_DATA(rgd->rd_bh[i]) + bits->bi_offset,
                     bits->bi_len, blk, GFS_BLKST_USEDMETA);
 
-    fblk = fs_bitfit((unsigned char *)BH_DATA(rgd->rd_bh[i]) + bits->bi_offset,
+    fblk = fs_bitfit(BH_DATA(rgd->rd_bh[i]) + bits->bi_offset,
 			     bits->bi_len, blk, GFS_BLKST_FREEMETA);
     if(ublk < fblk) {
 	    blk = ublk;
