@@ -1,3 +1,15 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) 2005 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include "lock_dlm.h"
 #include "ccs.h"
 
@@ -605,7 +617,7 @@ int loop(void)
 					process_client(i);
 			}
 
-			if (pollfd[i].revents & (POLLHUP | POLLERR | POLLNVAL)) {
+			if (pollfd[i].revents & POLLHUP) {
 				if (pollfd[i].fd == cman_fd) {
 					log_error("cman connection died");
 					exit_cman();
@@ -874,8 +886,7 @@ static void decode_arguments(int argc, char **argv)
 			break;
 
 		case 'V':
-			printf("gfs_controld %s (built %s %s)\n",
-				RELEASE_VERSION, __DATE__, __TIME__);
+			printf("gfs_controld (built %s %s)\n", __DATE__, __TIME__);
 			/* printf("%s\n", REDHAT_COPYRIGHT); */
 			exit(EXIT_SUCCESS);
 			break;
