@@ -1,3 +1,15 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) 2005-2007 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <sys/types.h>
 #include <asm/types.h>
 #include <sys/uio.h>
@@ -786,11 +798,15 @@ static int add_waiter(struct mountgroup *mg, struct resource *r,
 static void write_result(struct mountgroup *mg, struct dlm_plock_info *in,
 			 int rv)
 {
+	int err;
+
 	if (need_fsid_translation)
 		in->fsid = mg->associated_ls_id;
 
 	in->rv = rv;
-	write(control_fd, in, sizeof(struct dlm_plock_info));
+	err = write(control_fd, in, sizeof(struct dlm_plock_info));
+	if (err != sizeof(struct dlm_plock_info))
+		log_error("plock result write err %d errno %d", err, errno);
 }
 
 static void do_waiters(struct mountgroup *mg, struct resource *r)
@@ -1389,7 +1405,7 @@ static void _receive_sync(struct mountgroup *mg, char *buf, int len, int from)
 	}
 
 	if (hd->type == MSG_PLOCK_SYNC_LOCK)
-		add_lock(r, info.nodeid, info.owner, info.pid, info.ex, 
+		add_lock(r, info.nodeid, info.owner, info.pid, !info.ex, 
 			 info.start, info.end);
 	else if (hd->type == MSG_PLOCK_SYNC_WAITER)
 		add_waiter(mg, r, &info);
@@ -2223,7 +2239,7 @@ void retrieve_plocks(struct mountgroup *mg)
 			goto out_it;
 		}
 
-		if (!desc.sectionId.idLen)
+		if (!desc.sectionSize)
 			continue;
 
 		iov.sectionId = desc.sectionId;
@@ -2231,10 +2247,8 @@ void retrieve_plocks(struct mountgroup *mg)
 		iov.dataSize = desc.sectionSize;
 		iov.dataOffset = 0;
 
-		/* for debug print */
 		memset(&buf, 0, sizeof(buf));
 		snprintf(buf, SECTION_NAME_LEN, "%s", desc.sectionId.id);
-
 		log_group(mg, "retrieve_plocks: section size %llu id %u \"%s\"",
 			  (unsigned long long)iov.dataSize, iov.sectionId.idLen,
 			  buf);
@@ -2252,14 +2266,13 @@ void retrieve_plocks(struct mountgroup *mg)
 			goto out_it;
 		}
 
-		/* we'll get empty (zero length) sections for resources with
-		   no locks, which exist in ownership mode; the resource
-		   name and owner come from the section id */
-
 		log_group(mg, "retrieve_plocks: ckpt read %llu bytes",
 			  (unsigned long long)iov.readSize);
 		section_len = iov.readSize;
 
+		if (!section_len)
+		       continue;
+
 		if (section_len % sizeof(struct pack_plock)) {
 			log_error("retrieve_plocks: bad section len %d %s",
 				  section_len, mg->name);
