@@ -1,3 +1,21 @@
+/*
+  Copyright Red Hat, Inc. 2004
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2, or (at your option) any
+  later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; see the file COPYING.  If not, write to the
+  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
+  MA 02139, USA.
+*/
 #include <libxml/parser.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/xpath.h>
@@ -431,8 +449,7 @@ store_resource(resource_t **reslist, resource_t *newres)
    @return		newly allocated pointer to value or NULL if not found.
  */
 char *
-xpath_get_one(xmlDocPtr __attribute__ ((unused)) doc,
-	      xmlXPathContextPtr ctx, char *query)
+xpath_get_one(xmlDocPtr doc, xmlXPathContextPtr ctx, char *query)
 {
 	char *val = NULL, *ret = NULL;
 	xmlXPathObjectPtr obj;
@@ -627,11 +644,7 @@ act_dup(resource_act_t *acts)
 
 /* Copied from resrules.c -- _get_actions */
 void
-#ifndef NO_CCS
 _get_actions_ccs(int ccsfd, char *base, resource_t *res)
-#else
-_get_actions_ccs(int __attribute__((unused)) ccsfd, char *base, resource_t *res)
-#endif
 {
 	char xpath[256];
 	int idx = 0;
