@@ -1,3 +1,21 @@
+/*
+  Copyright Red Hat, Inc. 2004
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2, or (at your option) any
+  later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; see the file COPYING.  If not, write to the
+  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
+  MA 02139, USA.
+*/
 //#define DEBUG
 #include <rg_types.h>
 #include <resgroup.h>
@@ -66,7 +84,7 @@ forwarding_thread(void *arg)
 		clulog(LOG_ERR, "FW: Attempt to forward to invalid node ID\n");
        		goto out_fail;
 	}
-	if (rgs.rs_owner == (uint32_t)my_id()) {
+	if (rgs.rs_owner == my_id()) {
 		clulog(LOG_WARNING, "BUG! Attempt to forward to myself!\n");
        		goto out_fail;
 	}
@@ -90,7 +108,7 @@ forwarding_thread(void *arg)
 		       rgs.rs_owner, ctx);
 		goto out_fail;
 	}
-	if (msg_send(ctx, &msg, sizeof(msg)) < (int)sizeof(msg)) {
+	if (msg_send(ctx, &msg, sizeof(msg)) < sizeof(msg)) {
 		clulog(LOG_DEBUG, "FW: Failed to send message to %d CTX: %p\n",
 		       rgs.rs_owner, ctx);
 		goto out_fail;
@@ -192,7 +210,7 @@ forwarding_thread_v2(void *arg)
 
 	/* swap + send */
 	swab_SmMessageSt(msgp);
-	if (msg_send(ctx, msgp, sizeof(*msgp)) < (int)sizeof(*msgp)) {
+	if (msg_send(ctx, msgp, sizeof(*msgp)) < sizeof(*msgp)) {
 		clulog(LOG_DEBUG, "FW: Failed to send message to %d CTX: %p\n",
 		       target, ctx);
 		goto out_fail;
